@@ -96,9 +96,7 @@ impl<'a, I: Tokens> Parser<I> {
                 continue;
             }
             if comma > 0 {
-                // One comma is used for separating elements
-                let cnt = if elems.is_empty() { comma } else { comma - 1 };
-                elems.extend(iter::repeat(None).take(cnt));
+                elems.extend(iter::repeat(None).take(comma));
                 comma = 0;
             }
             let start = cur_pos!(self);
@@ -119,6 +117,10 @@ impl<'a, I: Tokens> Parser<I> {
             } else {
                 elems.push(self.parse_binding_element().map(Some)?);
             }
+
+            if !is!(self, ']') {
+                expect!(self, ',');
+            }
         }
 
         expect!(self, ']');
